@@ -74,6 +74,9 @@ setMethod(
   "cond_passage",
   signature(samc = "samc", init = "missing", origin = "missing", dest = "location"),
   function(samc, dest) {
+    .disable_conv(samc)
+    .disable_crw(samc)
+
     if (samc@clumps == -1)
       warning("Unknown number of clumps in data. If the function crashes, it may be due to the transition matrix being discontinuous.", call. = FALSE)
 
@@ -121,6 +124,9 @@ setMethod(
   "cond_passage",
   signature(samc = "samc", init = "missing", origin = "location", dest = "location"),
   function(samc, origin, dest) {
+    .disable_conv(samc)
+    .disable_crw(samc)
+
     if(length(origin) != length(dest))
       stop("The 'origin' and 'dest' parameters must have the same number of values", call. = FALSE)
 
